@@ -1030,9 +1030,75 @@ exit:
 	gpdf_control_enable_ui (control, TRUE);
 }
 
-/* Returns a filename in on-disk encoding (suitable for Xpdf code). */
-static gchar *
-gpdf_control_get_filename (BonoboControl *control)
+static gboolean
+overwrite_existing_file (GtkWindow *window, const gchar *file_name)
+{
+	GtkWidget *msgbox;
+	gchar *utf8_file_name;
+	gchar *msg;
+	AtkObject *obj;
+	gint ret;
+
+	utf8_file_name = g_filename_to_utf8 (file_name, -1, NULL, NULL, NULL);
+	msg = g_strdup_printf (_("A file named \"%s\" already exists."),
+			       utf8_file_name);
+	g_free (utf8_file_name);
+
+	msgbox = gpdf_hig_dialog_new (
+		GTK_STOCK_DIALOG_QUESTION, msg,
+		_("Do you want to replace it with the one you are saving?"),
+		FALSE);
+
+	g_free (msg);
+
+	gtk_dialog_add_button (GTK_DIALOG (msgbox), 
+			       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL);
+
+	gtk_dialog_add_button (GTK_DIALOG (msgbox),
+			       _("_Replace"), GTK_RESPONSE_YES);
+
+	gtk_window_set_transient_for (GTK_WINDOW (msgbox), window);
+
+	gtk_dialog_set_default_response (GTK_DIALOG (msgbox),
+					 GTK_RESPONSE_CANCEL);
+
+	obj = gtk_widget_get_accessible (msgbox);
+
+	if (GTK_IS_ACCESSIBLE (obj))
+		atk_object_set_name (obj, _("Question"));
+
+	ret = gtk_dialog_run (GTK_DIALOG (msgbox));
+	gtk_widget_destroy (msgbox);
+
+	return (ret == GTK_RESPONSE_YES);
+}
+
+static void
+save_error_dialog (GtkWindow *window, const gchar *file_name)
+{
+	char *msg;
+	GtkWidget *error_dialog;
+
+	msg = g_strdup_printf (_("The file could not be saved as \"%s\"."),
+			       file_name);
+
+	error_dialog = gpdf_hig_dialog_new (GTK_STOCK_DIALOG_ERROR,
+					    msg, NULL, FALSE);
+
+	g_free (msg);
+
+	gtk_dialog_add_button (GTK_DIALOG (error_dialog),
+			       GTK_STOCK_CLOSE,
+			       GTK_RESPONSE_CLOSE);
+
+	gtk_window_set_transient_for (GTK_WINDOW (error_dialog), window);
+
+	gtk_dialog_run (GTK_DIALOG (error_dialog));
+	gtk_widget_destroy (error_dialog);
+}
+
+static void
+gpdf_control_save_file (BonoboControl *control)
 {
 	GPdfControlPrivate *priv;
 	GtkFileFilter *pdf_filter, *all_filter;
@@ -1069,30 +1135,35 @@ gpdf_control_get_filename (BonoboControl *control)
 					  NULL);
 	gtk_widget_show (priv->file_chooser);
 
-	if (gtk_dialog_run(GTK_DIALOG (priv->file_chooser)) == GTK_RESPONSE_OK)
+	while (gtk_dialog_run (GTK_DIALOG (priv->file_chooser))
+	       == GTK_RESPONSE_OK) {
+
 		pathname = gtk_file_chooser_get_filename (
 			GTK_FILE_CHOOSER (priv->file_chooser));
 
+		if (g_file_test (pathname, G_FILE_TEST_EXISTS) &&  
+		    !overwrite_existing_file (GTK_WINDOW (priv->file_chooser),
+					      pathname))
+				continue;
+
+		if (gpdf_view_save_as (GPDF_VIEW (priv->gpdf_view), pathname))
+			break;
+		else
+			save_error_dialog (GTK_WINDOW (priv->file_chooser),
+					   pathname);    
+	}
+
 	gtk_widget_destroy (priv->file_chooser);
 	priv->file_chooser = NULL;
-	
-	return pathname; 
 }
 
 static void
 verb_SaveAs_cb (BonoboUIComponent *uic, gpointer user_data,
 		const char *cname)
 {
-	gchar *filename;
-	
 	g_return_if_fail (GPDF_IS_NON_NULL_CONTROL (user_data));
 
-	filename = gpdf_control_get_filename (BONOBO_CONTROL (user_data));
-	if (filename != NULL)
-	  gpdf_view_save_as (
-	      GPDF_VIEW (GPDF_CONTROL (user_data)->priv->gpdf_view),
-	      filename);
-	g_free (filename);
+	gpdf_control_save_file (BONOBO_CONTROL (user_data));
 }
 
 static void
