@@ -363,8 +363,7 @@ gpdf_view_link_action_uri (GPdfView *gpdf_view, LinkURI *link_uri)
 	}
 
 	action = gnome_vfs_mime_get_default_action (mime); 
-	if (gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK)
-	{
+	if (gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK) {
 		app = gnome_vfs_mime_get_default_application (mime); 
 		
 		if (!app || gnome_vfs_mime_application_launch (app, uris) != GNOME_VFS_OK) {
@@ -372,11 +371,19 @@ gpdf_view_link_action_uri (GPdfView *gpdf_view, LinkURI *link_uri)
 			/* Fallback: No component or app was found. Use a browser */
 			mime = "text/html";
 			action = gnome_vfs_mime_get_default_action (mime);
-			if (gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK)
-			{
+			if (gnome_vfs_mime_action_launch (action, uris) != GNOME_VFS_OK) {
 				app = gnome_vfs_mime_get_default_application (mime); 
-				if (!app || gnome_vfs_mime_application_launch (app, uris) != GNOME_VFS_OK) {
-			
+				if (!app) {
+					gchar *msg = 
+					  g_strdup_printf(_("No application was found to open the URI: %s"),
+							  (const gchar *)uris->data);
+					gpdf_control_private_error_dialog (gpdf_view->priv->parent, 
+									   _("Application Launch Failure!"),
+									   msg,
+									   TRUE,
+									   FALSE);
+					g_free (msg);
+				} else if (gnome_vfs_mime_application_launch (app, uris) != GNOME_VFS_OK) {
 					gchar *msg = 
 					  g_strdup_printf(_("Failed to launch %s for URI: %s"),
 							  app->name,
@@ -389,8 +396,7 @@ gpdf_view_link_action_uri (GPdfView *gpdf_view, LinkURI *link_uri)
 					g_free (msg);
 				}
 			}
-		}
-		else {
+		} else {
 			gchar *msg = g_strdup_printf ("URI '%s' launched with %s",
 						      (const gchar *)uris->data,
 						      app->name);
@@ -1479,21 +1485,24 @@ gpdf_view_set_pdf_doc (GPdfView *gpdf_view, PDFDoc *pdf_doc)
 }
 
 /* Expects pathname in on-disk encoding (like Xpdf's saveAs) */
-void
+gboolean
 gpdf_view_save_as (GPdfView *gpdf_view, const gchar *pathname)
 {
 	GPdfViewPrivate *priv; 
 	GString *filename = new GString(pathname); 
+	gboolean retval = FALSE;
 
-	g_return_if_fail (GPDF_IS_NON_NULL_VIEW (gpdf_view));
+	g_return_val_if_fail (GPDF_IS_NON_NULL_VIEW (gpdf_view), FALSE);
 
 	priv = gpdf_view->priv;
 
 	gpdf_control_private_set_wait_cursor (priv->parent);
 	
-	priv->pdf_doc->saveAs (filename);
-	
+	retval = priv->pdf_doc->saveAs (filename);
+
 	gpdf_control_private_reset_cursor (priv->parent);
+
+	return retval;
 }
 
 
